#!/usr/bin/env python3
import json
import sys
import os
import time
import zipfile
import datetime
import hashlib
import shutil

LOG_FILE = "/var/log/iq/data/meter.jsonlist.1"
ARCHIVE_DIR = "/var/log/iq/data"

def extract_unix_timestamp(line):
    try:
        obj = json.loads(line)
        ts = obj.get("timestamp")
        if isinstance(ts, (int, float)):
            return int(ts)
    except Exception:
        return None

def format_human(ts):
    return datetime.datetime.utcfromtimestamp(ts).isoformat().replace(":", "-")

def build_archive_name(first_ts, last_ts):
    human_range = "{}_to_{}".format(format_human(first_ts), format_human(last_ts))
    return "meter_{}_to_{}__{}.zip".format(first_ts, last_ts, human_range)

def fallback_name():
    ts = int(time.time())
    return "meter_fallback_{}.zip".format(ts)

def hash_file(path):
    h = hashlib.sha256()
    with open(path, "rb") as f:
        while True:
            chunk = f.read(8192)
            if not chunk:
                break
            h.update(chunk)
    return h.hexdigest()

def rotate():
    if not os.path.exists(LOG_FILE):
        print("Log file missing, skipping")
        return

    timestamp = int(time.time())
    rotated_file = "{}.{}.tmp".format(LOG_FILE, timestamp)

    # Переименовываем лог, mosquitto_sub продолжит писать в старый дескриптор
    os.rename(LOG_FILE, rotated_file)

    first_ts = None
    last_ts = None
    line_count = 0

    with open(rotated_file, 'rb') as f:
        for line in f:
            try:
                line = line.decode('utf-8').strip()
            except UnicodeDecodeError:
                continue
            if not line:
                continue
            ts = extract_unix_timestamp(line)
            if ts is not None:
                if first_ts is None:
                    first_ts = ts
                last_ts = ts
            line_count += 1

    if line_count == 0:
        os.remove(rotated_file)
        print("Rotated log is empty, deleted.")
        return

    if first_ts is None or last_ts is None or last_ts < first_ts:
        archive_name = fallback_name()
    else:
        archive_name = build_archive_name(first_ts, last_ts)

    archive_path = os.path.join(ARCHIVE_DIR, archive_name)

    meta = {
        "lines": line_count,
        "first_timestamp": first_ts,
        "last_timestamp": last_ts,
        "generated_at": timestamp,
        "sha256": hash_file(rotated_file)
    }

    with zipfile.ZipFile(archive_path, 'w', compression=zipfile.ZIP_DEFLATED) as zf:
        zf.write(rotated_file, arcname=os.path.basename(LOG_FILE))
        zf.writestr("meta.json", json.dumps(meta, indent=2))

    os.remove(rotated_file)
    print("Archived to {} with {} lines".format(archive_path, line_count))

def cleanup_archives(directory, max_size_mb=50, max_age_days=60):
    print("[cleanup] Checking archive directory...")

    files = [
        os.path.join(directory, f)
        for f in os.listdir(directory)
        if f.startswith("meter_") and f.endswith(".zip")
    ]

    now = time.time()
    max_age_seconds = max_age_days * 86400
    max_size_bytes = max_size_mb * 1024 * 1024

    total_size = 0
    to_delete_by_age = []

    file_stats = []
    for f in files:
        try:
            st = os.stat(f)
            total_size += st.st_size
            file_stats.append((f, st.st_mtime, st.st_size))

            if now - st.st_mtime > max_age_seconds:
                to_delete_by_age.append(f)
        except FileNotFoundError:
            continue

    # Удаляем по возрасту
    for f in to_delete_by_age:
        try:
            size = os.path.getsize(f)
            print("[cleanup] Removing old archive: {}".format(f))
            os.remove(f)
            total_size -= size
        except FileNotFoundError:
            continue

    # Пересчёт после удаления старых
    file_stats = [
        (f, mtime, size)
        for f, mtime, size in file_stats
        if os.path.exists(f) and f not in to_delete_by_age
    ]

    # Если всё ещё превышает лимит — удаляем старые
    if total_size > max_size_bytes:
        print("[cleanup] Total size exceeds {}MB: {:.2f}MB".format(max_size_mb, total_size/1024/1024))

        file_stats.sort(key=lambda x: x[1])  # сортируем по дате (старые сначала)

        for f, mtime, size in file_stats:
            print("[cleanup] Removing to reduce size: {}".format(f))
            os.remove(f)
            total_size -= size
            if total_size <= max_size_bytes:
                break

def cleanup_tmp_files(log_dir):
    for f in os.listdir(log_dir):
        if f.endswith(".tmp"):
            full_path = os.path.join(log_dir, f)
            try:
                os.remove(full_path)
                print("[cleanup] Removed leftover tmp file: {}".format(full_path))
            except Exception as e:
                print("[cleanup] Failed to remove tmp file: {} — {}".format(full_path, e))

def cleanup_old_logs(log_dir):
    for fname in os.listdir(log_dir):
        # удалить все файлы вида meter.jsonlist.NUMBER, кроме meter.jsonlist и meter.jsonlist.1
        if fname.startswith("meter.jsonlist.") and fname != "meter.jsonlist.1":
            full_path = os.path.join(log_dir, fname)
            try:
                os.remove(full_path)
                print("[cleanup] Deleted old log: {}".format(full_path))
            except Exception as e:
                print("[cleanup] Failed to delete {}: {}".format(full_path, e))

if __name__ == "__main__":
    cleanup_tmp_files(os.path.dirname(LOG_FILE))
    cleanup_archives(ARCHIVE_DIR)
    rotate()
    cleanup_old_logs(os.path.dirname(LOG_FILE))
